@@ -25,12 +25,15 @@ SEXP attr_hidden
     lme4_devianceSym,
     lme4_diagSym,
     lme4_dimsSym,
+    lme4_etaSym,
     lme4_factorSym,
     lme4_fixefSym,
     lme4_flistSym,
     lme4_gradCompSym,
     lme4_iSym,
+    lme4_muSym,
     lme4_ncSym,
+    lme4_offsetSym,
     lme4_pSym,
     lme4_permSym,
     lme4_rXySym,
@@ -40,6 +43,7 @@ SEXP attr_hidden
     lme4_uploSym,
     lme4_wrkresSym,
     lme4_wtsSym,
+    lme4_weightsSym,
     lme4_xSym,
     lme4_ySym;
 
